package com.github.confluencecamera;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.http.HttpVersion;
import org.apache.http.client.HttpClient;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.params.ConnManagerPNames;
import org.apache.http.conn.params.ConnPerRouteBean;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.SingleClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.xmlrpc.android.XMLRPCClient;
import org.xmlrpc.android.XMLRPCException;

import android.os.StrictMode;
import android.os.StrictMode.ThreadPolicy;
import android.util.Log;

import com.byarger.exchangeit.EasySSLSocketFactory;
import com.github.confluencecamera.model.Page;

public class ConfluenceProxy {
	private String username, password, url;
	private String token;
	private XMLRPCClient client;
	
	private static final String TAG = "ConfluenceProxy";
	
	public ConfluenceProxy() { 
		
	}
	
	public ConfluenceProxy(String url, String username, String password) {
		super();
		this.username = username;
		this.password = password;
		this.url = url;
		
		// remove trailing / if it has one
		url = url.replace("/$", "");
		
		client = new XMLRPCClient(url + "/rpc/xmlrpc");
		client.setClient(createClient());
	}

	public void login() throws XMLRPCException {
    	// TODO remove before release
    	ThreadPolicy tp = ThreadPolicy.LAX;
    	StrictMode.setThreadPolicy(tp); 
    	
		token = (String) client.call("confluence2.login", username, password);
		Log.d("ConfluenceProxy", token);
	}
	
	public void uploadImage(final byte[] image) throws XMLRPCException {
		Log.d(TAG, "Attempting to find page.");
		String spaceKey = "DEVTEAM";
		String pageTitle = "Image Capture Test";
		Page page = getPage(spaceKey, pageTitle);
		Log.d(TAG, "Uploading new attachment to page " + page.getId());
		
		Map<String,String> attachment = new HashMap<String,String>();
		attachment.put("fileName", username + " " + System.currentTimeMillis() + ".jpeg");
		attachment.put("fileSize", image.length + "");
		attachment.put("contentType", "image/jpeg");
		attachment.put("comment", "Uploaded via Android");
		
		client.call("confluence2.addAttachment", token, page.getId(), attachment, image);
		Log.d(TAG, "Attachment uploaded.");
	}
	
	@SuppressWarnings("unchecked")
	private Page getPage(String spaceKey, String pageTitle) throws XMLRPCException {
		Map<String,String> page = (Map<String, String>) client.call("confluence2.getPage", token, spaceKey, pageTitle);
		return unmarshalPage(page);
	}

	@SuppressWarnings("unchecked")
	public List<Page> searchPages(String query, int maxResults) throws XMLRPCException {
		List<Page> list = new LinkedList<Page>();
		Map<String,String> params = new HashMap<String,String>();
		params.put("type", "page");
		Object[] results = (Object[]) client.call("confluence2.search", token, query == null ? "Qiagram" : query, params, maxResults);
		
		Log.d("Confluence", "Found " + results.length + " pages matching \"" + query + "\"");
		
		for(Object result:results) {
			Map<String,String> map = (Map<String,String>) result;
			list.add(unmarshalPage(map));
		}
		
		return list;
	}
	
	private Page unmarshalPage(Map<String,String> map) {
		Page p = new Page();
		p.setId(map.get("id"));
		p.setTitle(map.get("title"));
		return p;
	}
	
	private HttpClient createClient() {
		SchemeRegistry schemeRegistry = new SchemeRegistry();
		schemeRegistry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
		schemeRegistry.register(new Scheme("https", new EasySSLSocketFactory(), 443));
		 
		HttpParams params = new BasicHttpParams();
		params.setParameter(ConnManagerPNames.MAX_TOTAL_CONNECTIONS, 30);
		params.setParameter(ConnManagerPNames.MAX_CONNECTIONS_PER_ROUTE, new ConnPerRouteBean(30));
		params.setParameter(HttpProtocolParams.USE_EXPECT_CONTINUE, false);
		HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
		 
		ClientConnectionManager cm = new SingleClientConnManager(params, schemeRegistry);
		return new DefaultHttpClient(cm, params);		
	}
	
	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}
}
