package com.github.confluencecamera;

import org.xmlrpc.android.XMLRPCException;

import android.app.Dialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

public class SettingsDialog extends Dialog {
	private EditText usernameField, passwordField, urlField;
	private SharedPreferences prefs;
	
	public SettingsDialog(Context context, SharedPreferences prefs) {
		super(context);
		this.prefs = prefs;
		setTitle("Settings");
		setContentView(R.layout.settings);
    	usernameField = (EditText) findViewById(R.id.username);
    	passwordField = (EditText) findViewById(R.id.password); 
    	urlField = (EditText) findViewById(R.id.confluenceUrl);
    	
    	Button testButton = (Button) findViewById(R.id.testConnectionButton);
    	testButton.setOnClickListener(new Button.OnClickListener() {
			@Override
			public void onClick(View v) {
				SettingsDialog.this.testConnection();
			}
		});
    	
    	Button saveButton = (Button) findViewById(R.id.saveButton);
    	saveButton.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View arg0) {
				SettingsDialog.this.saveSettings();
			}
    	});

    	Button cancelButton = (Button) findViewById(R.id.cancelButton);
    	cancelButton.setOnClickListener(new Button.OnClickListener() {
			@Override
			public void onClick(View v) {
				SettingsDialog.this.dismiss();
			}
		});
    	
    	loadSettings();
    }
    
	private void loadSettings() {
		usernameField.setText(prefs.getString("username", ""));
		passwordField.setText(prefs.getString("password", ""));
		urlField.setText(prefs.getString("url", "https://my.server/path/"));
	}
	
	private void saveSettings() {
		Editor editor = prefs.edit();
		editor.putString("username", usernameField.getText().toString());
		editor.putString("password", passwordField.getText().toString());
		editor.putString("url", urlField.getText().toString());
		editor.commit();
		
		Toast.makeText(this.getContext(), "Settings Saved.", Toast.LENGTH_SHORT).show();
		this.dismiss();
	}
	
    private void testConnection() {
    	ConfluenceProxy proxy = new ConfluenceProxy(urlField.getText().toString(), usernameField.getText().toString(), passwordField.getText().toString());
    	
    	try {
			proxy.login();
			Toast.makeText(this.getContext(), "Connection Successful!", Toast.LENGTH_SHORT).show();
    	} catch (XMLRPCException e) {
			Log.e("Settings", "Error logging in", e);
			Toast.makeText(this.getContext(), "Connection Failed.", Toast.LENGTH_SHORT).show();
		}
    	
    	Log.d("Settings", "Username: " + usernameField.getText());
    	Log.d("Settings", "Password: " + passwordField.getText());
    	Log.d("Settings", "URL: " + urlField.getText());
    }
}
